from __future__ import annotations

GET_TWITTER_POSTS_BY_IDS = "getTwitterPostsByIds"
GET_TWITTER_POSTS_BY_AUTHOR = "getTwitterPostsByAuthor"
SEARCH_TWITTER_POSTS = "getTwitterPostsByKeywords"
GET_TWITTER_RETWEETS = "getTwitterPostRetweets"
GET_TWITTER_QUOTES = "getTwitterPostQuotes"
GET_TWITTER_COMMENTS = "getTwitterPostComments"
GET_TWITTER_POST_INTERACTING_USERS = "getTwitterPostInteractingUsers"
COUNT_TWEETS = "countTweets"
GET_TWITTER_USER = "getTwitterUser"
SEARCH_TWITTER_USERS = "searchTwitterUsers"
GET_TWITTER_USER_CONNECTIONS = "getTwitterUserConnections"
GET_TWITTER_USERS_BY_KEYWORDS = "getTwitterUsersByKeywords"

GET_INSTAGRAM_POSTS_BY_IDS = "getInstagramPostsByIds"
GET_INSTAGRAM_POSTS_BY_USER = "getInstagramPostsByUser"
SEARCH_INSTAGRAM_POSTS = "getInstagramPostsByKeywords"
GET_INSTAGRAM_COMMENTS = "getInstagramCommentsByPostId"
GET_INSTAGRAM_USER = "getInstagramUser"
SEARCH_INSTAGRAM_USERS = "searchInstagramUsers"
GET_INSTAGRAM_USER_CONNECTIONS = "getInstagramUserConnections"
GET_INSTAGRAM_POST_INTERACTING_USERS = "getInstagramPostInteractingUsers"
GET_INSTAGRAM_USERS_BY_KEYWORDS = "getInstagramUsersByKeywords"

SEARCH_REDDIT_POSTS = "getRedditPostsByKeywords"
GET_REDDIT_POST_WITH_COMMENTS = "getRedditPostWithCommentsById"
SEARCH_REDDIT_COMMENTS = "getRedditCommentsByKeywords"
GET_REDDIT_USER = "getRedditUser"
SEARCH_REDDIT_USERS = "searchRedditUsers"
GET_REDDIT_USERS_BY_KEYWORDS = "getRedditUsersByKeywords"
SEARCH_REDDIT_SUBREDDITS = "searchRedditSubreddits"
GET_REDDIT_SUBREDDIT_WITH_POSTS = "getRedditSubredditWithPostsByName"
GET_REDDIT_SUBREDDITS_BY_KEYWORDS = "getRedditSubredditsByKeywords"
